//+------------------------------------------------------------------+
//|   AdaptiveLaguerreFilter.mq4
//+------------------------------------------------------------------+
#property indicator_chart_window

#property indicator_buffers 1
#property indicator_color1  DeepSkyBlue
#property indicator_width1  2

//
//
//
//
//

extern string TimeFrame   = "current time frame";
extern int    LookBack    = 20;
extern int    Median      = 5;
extern int    PriceType   = PRICE_MEDIAN;
extern bool   Interpolate = true;

//
//
//
//
//

double Filter[];
double Diff[];
double L0[];
double L1[];
double L2[];
double L3[];
double sortDiff[];

//
//
//
//
//

int    timeFrame;
string IndicatorFileName;
bool   ReturningBars;
bool   CalculatingAlf;

//+------------------------------------------------------------------+
//|
//+------------------------------------------------------------------+
//
//
//
//

int init() 
{

   IndicatorBuffers(6);
   SetIndexBuffer(0,Filter); SetIndexLabel(0,"ALF");
   SetIndexBuffer(1,Diff);
   SetIndexBuffer(2,L0);
   SetIndexBuffer(3,L1);
   SetIndexBuffer(4,L2);
   SetIndexBuffer(5,L3);
   ArrayResize(sortDiff,Median);

   //
   //
   //
   //
   //
         
      IndicatorFileName   = WindowExpertName();
      CalculatingAlf = (TimeFrame=="CalculateAlf"); if (CalculatingAlf) return(0);
      ReturningBars  = (TimeFrame=="returnBars");   if (ReturningBars)  return(0);
      timeFrame      = stringToTimeFrame(TimeFrame);
   
   //
   //
   //
   //
   //
   
   IndicatorShortName("Adaptive Laguerre filter "+timeFrameToString(timeFrame)+" ("+LookBack+","+Median+","+PriceType+")");
   return (0);
}
  
//+------------------------------------------------------------------+
//|
//+------------------------------------------------------------------+
//
//
//
//
//
//

int start()
{
   int counted_bars = IndicatorCounted();
   int i,limit;
    
   if (counted_bars<0) return(-1);
   if (counted_bars>0) counted_bars--;
      limit = MathMin(Bars-counted_bars,Bars-1);
           if (ReturningBars)  { Filter[0] = limit; return(0); }

   //
   //
   //
   //
   //
   
   if (CalculatingAlf || timeFrame==Period())
   {
      for (i=limit;i>=0;i--)
      {
         double price = iMA(NULL,0,1,0,MODE_SMA,PriceType,i);
         Diff[i] = MathAbs(price - Filter[i+1]);

         //
         //
         //
         //
         //
      
         double HH = Diff[i];
         double LL = Diff[i];
            for (int j=0; j<LookBack; j++)
            {
               if (Diff[i+j] > HH) {HH = Diff[i+j];}
               if (Diff[i+j] < LL) {LL = Diff[i+j];}
            }

            double alpha = 0.5;
            if (HH!=LL)
            {
               for (j=0;j<Median;j++) sortDiff[j] = (Diff[i+j]-LL)/(HH-LL);
               ArraySort(sortDiff,WHOLE_ARRAY,0,MODE_ASCEND);
         
               if (MathMod(Median,2.0) != 0)
                     alpha = sortDiff[Median/2];         
               else  alpha = (sortDiff[Median/2]+sortDiff[(Median/2)-1])/2;
            }
      
         //
         //
         //
         //
         //
      
         L0[i] = alpha*price + (1 - alpha)*L0[i+1];
         L1[i] = -(1 - alpha)*L0[i] + L0[i+1] + (1 - alpha)*L1[i+1];
         L2[i] = -(1 - alpha)*L1[i] + L1[i+1] + (1 - alpha)*L2[i+1];
         L3[i] = -(1 - alpha)*L2[i] + L2[i+1] + (1 - alpha)*L3[i+1];
         Filter[i] = (L0[i] + 2.0 * L1[i] + 2.0 * L2[i] + L3[i]) / 6.0;
      }
      return(0);
   }  
   
   //
   //
   //
   //
   //
   
   if (timeFrame > Period()) limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         Filter[i] = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateAlfr",LookBack,Median,PriceType,0,y);

         //
         //
         //
         //
         //
      
         if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
         if (!Interpolate) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            double factor = 1.0 / n;
            for(int k = 1; k < n; k++) Filter[i+k] = k*factor*Filter[i+n] + (1.0-k*factor)*Filter[i];
   }
   return (0);   
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}